using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace Schmiegekreis
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

            const int steps = 1000;
            PointF[] points = new PointF[steps];
            for (int i = 0; i < steps; i++)
            {
                double t = i / (steps - 1.0);
                PointF p = curve(t);
                points[i] = p;
            }
            e.Graphics.DrawLines(Pens.Blue, points);

            double t0 = trackBar1.Value / (double)trackBar1.Maximum;
            PointF p0 = curve(t0);
            e.Graphics.FillEllipse(Brushes.Red, p0.X - 2.0f, p0.Y - 2.0f, 4.0f, 4.0f);
            const double dt = 1e-2;
            PointF pR = curve(t0 + dt);
            double vx = (pR.X - p0.X) / dt;
            double vy = (pR.Y - p0.Y) / dt;
            PointF pL = curve(t0 - dt);
            double ax = (pL.X + pR.X - 2.0 * p0.X) / (dt * dt);
            double ay = (pL.Y + pR.Y - 2.0 * p0.Y) / (dt * dt);
            double factor = (vx*vx+vy*vy)/(ax*-vy+ay*vx);
            double dx = factor*-vy;
            double dy = factor*vx;
            e.Graphics.DrawLine(Pens.Green, p0.X, p0.Y, p0.X + (float)vx, p0.Y + (float)vy);
            double xM = p0.X+dx;
            double yM = p0.Y+dy;
            e.Graphics.FillEllipse(Brushes.Red, (float)(xM - 2.0), (float)(yM - 2.0), 4.0f, 4.0f);
            double r = Math.Sqrt(dx*dx+dy*dy);
            e.Graphics.DrawEllipse(Pens.Red, (float)(xM-r), (float)(yM-r), (float)(2.0*r), (float)(2.0*r));
        }

        private PointF curve(double t)
        {
            double x = 200.0 + 150.0 * Math.Cos(2.0 * Math.PI * t);
            double y = 200.0 + 50.0 * Math.Sin(2.0 * Math.PI * t);
            return new PointF((float)x, (float)y);
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            Invalidate();
        }
    }
}