using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.Direct3D;

namespace Schmiegekreis
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            base.OnPaint(e);
            e.Graphics.SmoothingMode = System.Drawing.Drawing2D.SmoothingMode.HighQuality;

            const int steps = 1000;
            PointF[] points = new PointF[steps];
            for (int i = 0; i < steps; i++)
            {
                double t = i / (steps - 1.0);
                Vector2 p = curve(t);
                points[i] = new PointF(p.X, p.Y);
            }
            e.Graphics.DrawLines(Pens.Blue, points);

            double t0 = trackBar1.Value / (double)trackBar1.Maximum;
            Vector2 p0 = curve(t0);
            e.Graphics.FillEllipse(Brushes.Red, p0.X - 2.0f, p0.Y - 2.0f, 4.0f, 4.0f);
            const float dt = 0.01f;
            Vector2 pR = curve(t0 + dt);
            Vector2 v = (pR - p0) * (1.0f/dt);
            Vector2 pL = curve(t0 - dt);
            Vector2 a = (pL + pR - 2.0f * p0) * (1.0f / (dt * dt));
            Vector2 vRot = new Vector2(-v.Y, v.X);
            float factor = v.LengthSq()/Vector2.Dot(a, vRot);
            Vector2 d = factor * vRot;
            e.Graphics.DrawLine(Pens.Green, p0.X, p0.Y, p0.X + v.X, p0.Y + v.Y);
            Vector2 m = p0 + d;
            e.Graphics.FillEllipse(Brushes.Red, (float)(m.X - 2.0), (float)(m.Y - 2.0), 4.0f, 4.0f);
            double r = d.Length();
            e.Graphics.DrawEllipse(Pens.Red, (float)(m.X - r), (float)(m.Y - r), (float)(2.0 * r), (float)(2.0 * r));
        }

        private Vector2 curve(double t)
        {
            double x = 200.0 + 150.0 * Math.Cos(2.0 * Math.PI * t);
            double y = 200.0 + 50.0 * Math.Sin(2.0 * Math.PI * t);
            return new Vector2((float)x, (float)y);
        }

        private void trackBar1_Scroll(object sender, EventArgs e)
        {
            Invalidate();
        }
    }
}